#!/bin/sh

#
# complete benchmark
#

# sqlite settings
SLDB=sl.db # database file

# postgresql settings
PGDATABASE=template1
unset PGHOST
PGUSER=postgres
PGPASSWORD=""
PATH=/usr/local/pgsql/bin:$PATH
export PGUSER PATH

# mysql settings
MSUSER=root
MSPASSWORD=""
MSDB=testdb
PATH=/usr/local/mysql/bin:$PATH
export PATH

#
# command line arguments
#-----------------------------------------------------------
USAGEMSG="Usage:\n\
\t`basename $0` [-n <times>] [-v] <what>\n\
Possible values for <what>:\n\
\tall      - test all databases\n\
\tsqlite   - test sqlite\n\
\tpostgres - test postgres"

WHAT=""
WHATALL="sqlite postgres mysql"
TIMES=1
VERBOSE=""

while [ $# -gt 0 ]
do
	case "$1" in
		all)      WHAT="$WHATALL";;
		sqlite)   WHAT="$WHAT $1";;
		postgres) WHAT="$WHAT $1";;
		mysql)    WHAT="$WHAT $1";;
		-n)       TIMES=$2; shift;;
		-v)       VERBOSE="-verbose";;
		*)        echo -e "$USAGEMSG"; exit 1;;
	esac
	shift
done
test -z "$WHAT" -o -z "$TIMES" && { echo -e "$USAGEMSG"; exit 1; }
echo $TIMES | grep -q '[^0-9]' && { echo -e "$USAGEMSG"; exit 1; }

#
# Loop over benchmarks
#-----------------------------------------------------------

for what in $WHAT
do

  # benchmark for sqlite
  if [ $what = sqlite ]
  then
	  n=0
	  echo
	  echo "Results for SQLite:"
	  echo "-------------------"
	  while [ $n -lt $TIMES ]
	  do
		n=`expr $n + 1`
		sqlite $SLDB <<EOF
.read create.sql
.exit
EOF
		benchsl $VERBOSE $SLDB
		sqlite $SLDB <<EOF
.read drop.sql
.exit
EOF
	  done

  fi

  # benchmark for postgresl
  if [ $what = postgres ]
  then
	  n=0
	  echo
	  echo "Results for PostgreSQL:"
	  echo "-----------------------"
	  while [ $n -lt $TIMES ]
	  do
		n=`expr $n + 1`
		psql -q -f create.sql -d $PGDATABASE 2>&1 | grep -v NOTICE
		benchpg $VERBOSE -u $PGUSER -p "$PGPASSWORD" $PGDATABASE
		psql -q -f drop.sql -d $PGDATABASE
	  done
  fi

  # benchmark for mysql
  if [ $what = mysql ]
  then
	  n=0
	  echo
	  echo "Results for MySQL:"
	  echo "-----------------------"
	  while [ $n -lt $TIMES ]
	  do
		n=`expr $n + 1`
		mysql --user=$MSUSER $MSDB < createmysql.sql
		benchms -u $MSUSER -p "$MSPASSWORD" $VERBOSE $MSDB
		mysql --user=$MSUSER $MSDB < drop.sql
	  done
  fi

done
