//
// Database interface class implementation for sqlite
//
// Author: Christoph Dalitz, June 2004
//

#include "dbi.h"

#include <mysql.h>

#include <stdlib.h>
#include <unistd.h>

class DbiMS : public Dbi {

public:
  DbiMS();
  ~DbiMS();
  virtual int Login(const char* dbname, const char* user, const char* passwd);
  virtual void Logout();
  virtual int Select(const char* sql, StringList* res);
  virtual int NonSelect(const char* sql);

private:
  MYSQL *db;
};

// Constructor
DbiMS::DbiMS() {
  lasterror = "";
  db = NULL;
}

// Destructor
DbiMS::~DbiMS() {
  if (db)
    this->Logout();
}

// DB-Login
int DbiMS::Login(const char* dbname, const char* user, const char* passwd) {
  char* dberr;
  lasterror = "";
  const char *mspwd, *msusr;

  if (!user || !*user)
    msusr = NULL;
  else
    msusr = user;
  if (!passwd || !*passwd)
    mspwd = NULL;
  else
    mspwd = passwd;

  db = mysql_init(NULL);
  if (!mysql_real_connect(db, NULL /*host*/, msusr, mspwd, dbname,
                          0 /*port*/, NULL /*unix_socket*/, 0)) {
    lasterror = mysql_error(db);
    return 1;
  }
  return 0;
}

// DB-Logout
void DbiMS::Logout() {
  lasterror = "";
  if (db) {
    mysql_close(db);
    db = NULL;
  }
  return;
}

// execute Select-Statement
int DbiMS::Select(const char* sql, StringList* res) {

  MYSQL_ROW dbrow;
  MYSQL_RES *dbres;
  int i,n;
  int first = 1;

  lasterror = "";
  res->clear();

  if (mysql_query(db, sql)) {
    lasterror = mysql_error(db);
    return 1;
  }
  dbres = mysql_use_result(db);
  // for some reason the mysql docs say that we MUST fetch all rows (?!)
  while (dbrow = mysql_fetch_row(dbres)) {
    if (first) {
      n = mysql_num_fields(dbres);
      for(i = 0; i < n; i++) {
        res->push_back(dbrow[i]);
      }
      first = 0;
    }
  }
  mysql_free_result(dbres);

  return 0;
}

// execute Non-Select-Statement
int DbiMS::NonSelect(const char* sql) {
  lasterror = "";
  if (mysql_query(db, sql)) {
    lasterror = mysql_error(db);
    return 1;
  }
  return 0;
}

// implementation of constructor for generic class
Dbi* ConstructDbi() {
  return (Dbi*) new DbiMS();
}
