//
// Database interface class implementation for postgresql
//
// Author: Christoph Dalitz, June 2004
//

#include "dbi.h"

#include <libpq-fe.h>
#include <stdlib.h>

class DbiPG : public Dbi {

public:
  DbiPG();
  ~DbiPG();
  virtual int Login(const char* dbname, const char* user, const char* passwd);
  virtual void Logout();
  virtual int Select(const char* sql, StringList* res);
  virtual int NonSelect(const char* sql);

private:
  PGconn *db;
};

// Constructor
DbiPG::DbiPG() {
  lasterror = "";
  db = NULL;
}

// Destructor
DbiPG::~DbiPG() {
  if (db)
    this->Logout();
}

// DB-Login
int DbiPG::Login(const char* dbname, const char* user, const char* passwd) {
  string connstr;
  lasterror = "";
  connstr = (string)"dbname=" + dbname;
  if (user && *user)
    connstr += (string)" user=" + user;
  if (passwd && *passwd)
    connstr += (string)" password=" + passwd;
  db = PQconnectdb(connstr.c_str());
  if (PQstatus(db) == CONNECTION_BAD) {
    lasterror = PQerrorMessage(db);
    db = NULL;
    return 1;
  }
  return 0;
}

// DB-Logout
void DbiPG::Logout() {
  lasterror = "";
  if (db) {
    PQfinish(db);
    db = NULL;
  }
  return;
}

// execute Select-Statement
int DbiPG::Select(const char* sql, StringList* res) {
  PGresult *dbres;
  int i,n;

  lasterror = "";
  res->clear();

  dbres = PQexec(db, sql);
  if (PQresultStatus(dbres) != PGRES_TUPLES_OK) {
    lasterror = PQresultErrorMessage(dbres);
    PQclear(dbres);
    return 1;
  }

  if (PQntuples(dbres) > 0) {
    n = PQnfields(dbres);
    for (int i=0; i < n; i++) {
      res->push_back(PQgetvalue(dbres, 0, i));
    }
  }

  PQclear(dbres);

  return 0;
}

// execute Non-Select-Statement
int DbiPG::NonSelect(const char* sql) {
  PGresult *res;
  int rc = 0;

  lasterror = "";
  res = PQexec(db, sql);
  if (PGRES_COMMAND_OK != PQresultStatus(res)) {
    lasterror = PQresultErrorMessage(res);
    rc = 1;
  }
  PQclear(res);
  return rc;
}

// implementation of constructor for generic class
Dbi* ConstructDbi() {
  return (Dbi*) new DbiPG();
}
