//
// Database interface class implementation for sqlite
//
// Author: Christoph Dalitz, June 2004
//

#include "dbi.h"

#include <sqlite.h>

#include <stdlib.h>
#include <unistd.h>

class DbiSL : public Dbi {

public:
  DbiSL();
  ~DbiSL();
  virtual int Login(const char* dbname, const char* user, const char* passwd);
  virtual void Logout();
  virtual int Select(const char* sql, StringList* res);
  virtual int NonSelect(const char* sql);

private:
  sqlite *db;
};

// Constructor
DbiSL::DbiSL() {
  lasterror = "";
  db = NULL;
}

// Destructor
DbiSL::~DbiSL() {
  if (db)
    this->Logout();
}

// DB-Login
int DbiSL::Login(const char* dbname, const char* user, const char* passwd) {
  char* dberr;
  lasterror = "";

  // check for file existence
  if (0 != access(dbname, R_OK)) {
    lasterror = (string)"database file '" + dbname + "' not found";
    return 1;
  }

  db = sqlite_open(dbname, 0, &dberr);
  if (!db) {
    lasterror = dberr;
    free(dberr);
    return 1;
  }
  return 0;
}

// DB-Logout
void DbiSL::Logout() {
  lasterror = "";
  if (db) {
    sqlite_close(db);
    db = NULL;
  }
  return;
}

// execute Select-Statement
int DbiSL::Select(const char* sql, StringList* res) {
  sqlite_vm *dbcursor;
  char *dberr = 0;
  char *ptail;

  lasterror = "";
  res->clear();

  if (SQLITE_OK != sqlite_compile(db, sql, (const char**)&ptail,
                                  &dbcursor, &dberr)) {
    lasterror = dberr;
    free(dberr);
    return 1;
  } else {
    int ncols, i;
    char **value, **name;
    if (SQLITE_ROW == sqlite_step(dbcursor, &ncols,
                                     (const char***)&value,
                                     (const char***)&name)) {
      for (i=0; i<ncols; i++) {
        if (value[i])
          res->push_back(value[i]);
        else
          res->push_back("NULL");
      }
    }
    sqlite_finalize(dbcursor, NULL);
  }    

  return 0;
}

// execute Non-Select-Statement
int DbiSL::NonSelect(const char* sql) {
  char* dberr;
  lasterror = "";
  if (SQLITE_OK != sqlite_exec(db, sql, NULL, NULL, &dberr)) {
    lasterror = dberr;
    free(dberr);
    return 1;
  }
  return 0;
}

// implementation of constructor for generic class
Dbi* ConstructDbi() {
  return (Dbi*) new DbiSL();
}
